

/*
I got a question on a forum earlier that I can't answer. Thought i'd see 
if anyone here can explain what's happening.

Fix expected in SP2
*/

CREATE TABLE Employees(
 ID INT NOT NULL IDENTITY,
 ManagerID INT NULL,
 Department VARCHAR(2) NOT NULL
)
GO

Insert into  Employees (ManagerID, Department)
select CASE column_id WHEN 1 then NULL ELSE column_id END, 'ab'
FROM master.sys.columns

CREATE CLUSTERED INDEX PK_Employee ON Employees ( ID )
GO

CREATE INDEX IX_0 ON Employees ( Department )
WHERE ManagerID IS NULL
GO

SELECT ID
FROM Employees
WHERE ManagerID IS NULL

-- 8622 Could not produce a plan
SELECT ID
FROM Employees with (index= IX_0)
WHERE ManagerID IS NOT NULL

/*
The predicate on the filtered index is the same as is on the query. The 
index is covering (since ID is the clustering key). I would have 
expected that the query would simply do an index scan on IX_0

If run just like that, it does a clustered index scan. If the filtered 
index is forced via a hint, the exec plan shows a index scan and a key 
lookup, but the output list of the key lookup is empty.

Why is that key lookup necessary? What am I missing?
*/

--To make matters more interesting, if we create a second filtered index: 
CREATE INDEX IX_1 ON Employees ( Department )
WHERE ManagerID IS NOT NULL
GO

--And flip the predicate on the query around to match the new index
SELECT ID
FROM Employees with (index= IX_1)
WHERE ManagerID IS NOT NULL

/*
The filtered index still isn't picked by default, without a hint it's 
still a clustered index scan, but when it's forced there's no key 
lookup. Just the index scan.

So why would the key lookup be there when the predicate and index 
filters are IS NULL but not when the predicate and index filters are IS 
NOT NULL

Original thread here, in case anyone wants to reply directly there.
http://www.sqlservercentral.com/Forums/Topic708894-392-1.aspx
*/